# 40MHz & payload I/O clocks derived from board-local source (for tests without external clock source)
create_generated_clock -name clk_40_pseudo_i [get_pins ttc/bufgce_clk_40_rx/O]
create_generated_clock -name clk_40_pseudo   -source [get_pins ttc/clocks/mmcm/CLKIN2] [get_pins ttc/clocks/mmcm/CLKOUT1]
create_generated_clock -name clk_payload_pseudo -source [get_pins ttc/clocks/mmcm/CLKIN2] [get_pins ttc/clocks/mmcm/CLKOUT3]

# Names of the LHC-synchronised derived clocks that are used by payload and infra (outside of TTC block)
lappend lMainInternalClocks clk_40_pseudo clk_payload_pseudo


if {[llength [get_pins -quiet ttc/gen_master_legacy.osc_clock/mmcm/CLKOUT1]] != 0} {
    create_generated_clock -name clk_40_extern_i0 [get_pins ttc/gen_master_legacy.osc_clock/gen_mmcm/mmcm/CLKOUT1]
}

if {[llength [get_pins -quiet ttc/gen_master_tcds2.tcds2/tcds2_interface/bufgce_clk_40_rx/O]] != 0} {
    create_generated_clock -name clk_40_extern_i1 [get_pins ttc/gen_master_tcds2.tcds2/tcds2_interface/bufgce_clk_40_rx/O]
}

foreach i [list 0 1] {
    if {[llength [get_clocks -quiet clk_40_extern_i$i]] != 0 } {
        create_generated_clock -name clk_40_extern$i -master_clock [get_clocks clk_40_extern_i$i] [get_pins ttc/clocks/mmcm/CLKOUT1]
        # Payload I/O clock derived from external oscillator
        create_generated_clock -name clk_payload_extern$i -master_clock [get_clocks clk_40_extern_i$i] [get_pins ttc/clocks/mmcm/CLKOUT3]

        lappend lMainInternalClocks clk_40_extern_i$i clk_40_extern$i clk_payload_extern$i
    }
}


# Clock groups: Asynchronous
puts "lMainInternalClocks = $lMainInternalClocks"
set_clock_groups -asynch -group [get_clocks axi_clk] -group [get_clocks -include_generated_clocks ipbus_clk]
set_clock_groups -asynch -group [get_clocks -include_generated_clocks ipbus_clk] -group $lMainInternalClocks

# Clock groups: Logically exclusive
# The 40MHz, 160MHz and payload clocks used across the chip are generated by an MMCM with two clock sources:
#  * an externally-provided LHC clock (typically from TCDS), which itself could come from one of two interfaces (clk_40_externN); and
#  * an internally-generated 'pseudo-LHC' clock (clk_40_pseudo)
# Must set the copies of the clocks generated from those sources as logically exclusive
foreach lMasterA [list clk_40_pseudo_i clk_40_pseudo_i clk_40_extern_i0] lMasterB [list clk_40_extern_i0 clk_40_extern_i1 clk_40_extern_i1] {
    if {[llength [get_clocks -quiet [list $lMasterA $lMasterB]]] == 2} {
        puts "Declaring clocks derived from $lMasterA and $lMasterB as logically exclusive"
        set_clock_groups -logically_exclusive -group [get_clocks -filter "MASTER_CLOCK == $lMasterA"] -group [get_clocks -filter "MASTER_CLOCK == $lMasterB"]
    }
}


# Remove timing constraints for heartbeat LED output port
set_false_path -to [get_ports -regexp "(.+_led|led_.+)"]

# Print clock report for the record (in case build fails, or need to debug clock-related issues)
report_clocks

